"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AntSqlModelGenerator_1 = require("../../model/AntSqlModelGenerator");
const DBTestManager_1 = require("./DBTestManager");
const MAX_SAFE_TIMEOUT = Math.pow(2, 31) - 1;
const namedModelGenerator = (keyGen, autoGeneratedId = false) => AntSqlModelGenerator_1.modelGenerator(keyGen, ['name'], autoGeneratedId ? 'id' : null);
const tableGeneratorColumnId = { name: 'id', type: 'number' };
const tableGeneratorOtherColumns = { name: 'string' };
class AntSqlSecondaryEntityManagerTest {
    constructor(beforeAllPromise, dbConnection, dbAlias, secondaryEntityManagerGenerator) {
        this._beforeAllPromise = beforeAllPromise;
        this._dbConnection = dbConnection;
        this._dbTestManager = new DBTestManager_1.DBTestManager();
        this._declareName = AntSqlSecondaryEntityManagerTest.name + '/' + dbAlias;
        this._secondaryEntityManagerGenerator = secondaryEntityManagerGenerator;
    }
    performTests() {
        describe(this._declareName, () => {
            this._itMustBeInitializable();
            this._itMustDeleteAnEntity();
            this._itMustDeleteMultipleEntities();
            this._itMustDeleteZeroEntities();
            this._itMustGetAnElementById();
            this._itMustGetAnUnexistingElementById();
            this._itMustGetMultipleElementsByIds();
            this._itMustGetMultipleElementsByIdsOrderedAsc();
            this._itMustInsertAnEntity();
            this._itMustInsertAnEntityWithAutoIncrementStrategy();
            this._itMustInsertMultieplEntities();
            this._itMustInsertMultipleEntitiesWithAutoIncrementStrategy();
            this._itMustInsertZeroEntities();
            this._itMustUpdateAnEntity();
            this._itMustUpdateMultipleEntities();
            this._itMustUpdateZeroEntities();
        });
    }
    _itMustBeInitializable() {
        const itsName = 'mustBeInitializable';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = AntSqlModelGenerator_1.modelGenerator({ prefix: prefix });
            expect(() => {
                this._secondaryEntityManagerGenerator(model, this._dbConnection);
            }).not.toThrowError();
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustDeleteAnEntity() {
        const itsName = 'mustDeleteAnEntity';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = AntSqlModelGenerator_1.modelGenerator({ prefix: prefix });
            const entity = { id: 0 };
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId);
            await this._dbTestManager.insert(this._dbConnection, model.tableName, entity);
            const secondaryEntityManager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            await secondaryEntityManager.delete(entity.id);
            expect(await secondaryEntityManager.getById(entity.id)).toBeNull();
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustDeleteMultipleEntities() {
        const itsName = 'mustDeleteMultipleEntities';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = AntSqlModelGenerator_1.modelGenerator({ prefix: prefix });
            const entity = { id: 0 };
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId);
            await this._dbTestManager.insert(this._dbConnection, model.tableName, entity);
            const secondaryEntityManager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            await secondaryEntityManager.mDelete([entity.id]);
            expect(await secondaryEntityManager.getById(entity.id)).toBeNull();
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustDeleteZeroEntities() {
        const itsName = 'mustDeleteZeroEntities';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = namedModelGenerator({ prefix: prefix });
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId, tableGeneratorOtherColumns);
            const secondaryEntityManager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            expectAsync(secondaryEntityManager.mDelete(new Array())).toBeResolved();
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustGetAnElementById() {
        const itsName = 'mustGetAnElementById';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = AntSqlModelGenerator_1.modelGenerator({ prefix: prefix });
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId);
            const entity = { id: 2 };
            await this._dbTestManager.insert(this._dbConnection, model.tableName, entity);
            const manager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            const entityFound = await manager.getById(entity.id);
            expect(Object.assign({}, entityFound)).toEqual(entity);
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustGetAnUnexistingElementById() {
        const itsName = 'mustGetAnUnexistingElementById';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = AntSqlModelGenerator_1.modelGenerator({ prefix: prefix });
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId);
            const manager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            const unexistingId = 1;
            const entityFound = await manager.getById(unexistingId);
            expect(entityFound).toBeNull();
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustGetMultipleElementsByIds() {
        const itsName = 'mustGetMultipleElementsByIds';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = AntSqlModelGenerator_1.modelGenerator({ prefix: prefix });
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId);
            const entity1 = { id: 1 };
            const entity2 = { id: 2 };
            await this._dbTestManager.insert(this._dbConnection, model.tableName, entity1);
            await this._dbTestManager.insert(this._dbConnection, model.tableName, entity2);
            const manager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            const entitiesFound = await manager.getByIds([
                entity1.id,
                entity2.id,
            ]);
            const entitiesFoundAsObjects = entitiesFound.map((entity) => (Object.assign({}, entity)));
            expect(entitiesFoundAsObjects).toContain(entity1);
            expect(entitiesFoundAsObjects).toContain(entity2);
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustGetMultipleElementsByIdsOrderedAsc() {
        const itsName = 'mustGetMultipleElementsByIdsOrderedAsc';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = AntSqlModelGenerator_1.modelGenerator({ prefix: prefix });
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId);
            const entity1 = { id: 1 };
            const entity2 = { id: 2 };
            await this._dbTestManager.insert(this._dbConnection, model.tableName, entity1);
            await this._dbTestManager.insert(this._dbConnection, model.tableName, entity2);
            const manager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            const entitiesFound = await manager.getByIdsOrderedAsc([
                entity1.id,
                entity2.id,
            ]);
            const entitiesFoundAsObjects = entitiesFound.map((entity) => (Object.assign({}, entity)));
            expect(entitiesFoundAsObjects).toEqual([entity1, entity2]);
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustInsertAnEntity() {
        const itsName = 'mustInsertAnEntity';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = namedModelGenerator({ prefix: prefix });
            const entity = { id: 0, name: 'name' };
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId, tableGeneratorOtherColumns);
            const secondaryEntityManager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            await secondaryEntityManager.insert(entity);
            const insertedEntity = await secondaryEntityManager.getById(entity.id);
            expect(Object.assign({}, insertedEntity)).toEqual(entity);
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustInsertAnEntityWithAutoIncrementStrategy() {
        const itsName = 'mustInsertAnEntityWithAutoIncrementStrategy';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = namedModelGenerator({ prefix: prefix }, true);
            const entity = { id: undefined, name: 'name' };
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, { name: 'id', type: 'increments' }, tableGeneratorOtherColumns);
            const secondaryEntityManager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            await secondaryEntityManager.insert(entity);
            const entityFound = await secondaryEntityManager.getById(entity.id);
            expect(entityFound.id).not.toBeUndefined();
            expect(entityFound.id).not.toBeNull();
            expect(entityFound.name).toBe(entity.name);
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustInsertMultieplEntities() {
        const itsName = 'mustInsertMultieplEntities';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = namedModelGenerator({ prefix: prefix });
            const entity = { id: 0, name: 'name' };
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId, tableGeneratorOtherColumns);
            const secondaryEntityManager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            await secondaryEntityManager.mInsert([entity]);
            const insertedEntity = await secondaryEntityManager.getById(entity.id);
            expect(Object.assign({}, insertedEntity)).toEqual(entity);
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustInsertMultipleEntitiesWithAutoIncrementStrategy() {
        const itsName = 'mustInsertMultipleEntitiesWithAutoIncrementStrategy';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = namedModelGenerator({ prefix: prefix }, true);
            const entity = { id: undefined, name: 'name' };
            const entity2 = { id: undefined, name: 'name2' };
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, { name: 'id', type: 'increments' }, tableGeneratorOtherColumns);
            const secondaryEntityManager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            await secondaryEntityManager.mInsert([entity, entity2]);
            const entityFound = await secondaryEntityManager.getById(entity.id);
            const entityFound2 = await secondaryEntityManager.getById(entity2.id);
            expect(typeof entityFound.id).toBe('number');
            expect(entityFound.name).toBe(entity.name);
            expect(typeof entityFound2.id).toBe('number');
            expect(entityFound2.name).toBe(entity2.name);
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustInsertZeroEntities() {
        const itsName = 'mustInsertZeroEntities';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = namedModelGenerator({ prefix: prefix });
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId, tableGeneratorOtherColumns);
            const secondaryEntityManager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            expectAsync(secondaryEntityManager.mInsert(new Array())).toBeResolved();
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustUpdateAnEntity() {
        const itsName = 'mustUpdateAnEntity';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = namedModelGenerator({ prefix: prefix });
            const entity = { id: 0, name: 'name' };
            const entityAfter = { id: 0, name: 'nameAfter' };
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId, tableGeneratorOtherColumns);
            await this._dbTestManager.insert(this._dbConnection, model.tableName, entity);
            const secondaryEntityManager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            await secondaryEntityManager.update(entityAfter);
            const entityUpdated = await secondaryEntityManager.getById(entity.id);
            expect(Object.assign({}, entityUpdated)).toEqual(entityAfter);
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustUpdateMultipleEntities() {
        const itsName = 'mustUpdateMultipleEntities';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = namedModelGenerator({ prefix: prefix });
            const entity = { id: 0, name: 'name' };
            const entityAfter = { id: 0, name: 'nameAfter' };
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId, tableGeneratorOtherColumns);
            await this._dbTestManager.insert(this._dbConnection, model.tableName, entity);
            const secondaryEntityManager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            await secondaryEntityManager.mUpdate([entityAfter]);
            const updatedEntity = await secondaryEntityManager.getById(entity.id);
            expect(Object.assign({}, updatedEntity)).toEqual(entityAfter);
            done();
        }, MAX_SAFE_TIMEOUT);
    }
    _itMustUpdateZeroEntities() {
        const itsName = 'mustUpdateZeroEntities';
        const prefix = this._declareName + '/' + itsName + '/';
        it(itsName, async (done) => {
            await this._beforeAllPromise;
            const model = namedModelGenerator({ prefix: prefix });
            await this._dbTestManager.createTable(this._dbConnection, model.tableName, tableGeneratorColumnId, tableGeneratorOtherColumns);
            const secondaryEntityManager = this._secondaryEntityManagerGenerator(model, this._dbConnection);
            expectAsync(secondaryEntityManager.mUpdate(new Array())).toBeResolved();
            done();
        }, MAX_SAFE_TIMEOUT);
    }
}
exports.AntSqlSecondaryEntityManagerTest = AntSqlSecondaryEntityManagerTest;
//# sourceMappingURL=AntSqlSecondaryEntityManagerTest.js.map