"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AntMySqlSecondaryEntityManager_1 = require("../../../persistence/secondary/AntMySqlSecondaryEntityManager");
const AntSQLiteSecondaryEntityManager_1 = require("../../../persistence/secondary/AntSQLiteSecondaryEntityManager");
const AntSqlSecondaryEntityManager_1 = require("../../../persistence/secondary/AntSqlSecondaryEntityManager");
const KnexDriver_1 = require("../../../persistence/secondary/KnexDriver");
class DBTestManager {
    async createTable(dbConnection, name, primaryKeyColumn, otherColumns = {}) {
        const tableExists = await dbConnection.schema.hasTable(name);
        if (tableExists) {
            throw new Error(`Table ${name} already exists.`);
        }
        await dbConnection.schema.createTable(name, (table) => {
            const addColumn = (name, type) => {
                if ('number' === type) {
                    table.integer(name);
                }
                else if ('string' === type) {
                    table.string(name);
                }
                else if ('increments' === type) {
                    table.increments(name);
                }
            };
            addColumn(primaryKeyColumn.name, primaryKeyColumn.type);
            for (const columnName in otherColumns) {
                if (otherColumns.hasOwnProperty(columnName)) {
                    addColumn(columnName, otherColumns[columnName]);
                }
            }
        });
        return name;
    }
    async deleteAllTables(dbConnection) {
        const tables = await this.listTables(dbConnection);
        return Promise.all(tables.map((table) => dbConnection.schema.dropTable(table)));
    }
    insert(dbConnection, table, entity) {
        return dbConnection(table).insert(entity);
    }
    createDatabaseIfNotExists(knex, dbName) {
        let query;
        switch (knex.client.driverName) {
            case KnexDriver_1.KnexDriver.MSSQL:
                query =
                    `IF  NOT EXISTS (SELECT * FROM sys.databases WHERE name = N'${dbName}')
BEGIN
    CREATE DATABASE [${dbName}]
END`;
                break;
            default:
                throw new Error(`Driver "${knex.client.driverName}" not supported`);
        }
        return knex.raw(query);
    }
    getSecondaryEntityManagerGenerator(knex) {
        switch (knex.client.driverName) {
            case KnexDriver_1.KnexDriver.MYSQL:
            case KnexDriver_1.KnexDriver.MYSQL2:
                return (model, knex) => new AntMySqlSecondaryEntityManager_1.AntMySqlSecondaryEntityManager(model, knex);
            case KnexDriver_1.KnexDriver.SQLITE3:
                return (model, knex) => new AntSQLiteSecondaryEntityManager_1.AntSQLiteSecondaryEntityManager(model, knex);
            default:
                return (model, knex) => new AntSqlSecondaryEntityManager_1.AntSqlSecondaryEntityManager(model, knex);
        }
    }
    listTables(knex) {
        let knexQuery;
        switch (knex.client.driverName) {
            case KnexDriver_1.KnexDriver.MSSQL:
                knexQuery = knex('information_schema.tables')
                    .select('table_name')
                    .where('TABLE_TYPE', 'BASE TABLE')
                    .andWhere('table_catalog', knex.client.database());
                break;
            case KnexDriver_1.KnexDriver.MYSQL:
            case KnexDriver_1.KnexDriver.MYSQL2:
                knexQuery = knex('information_schema.tables')
                    .select('TABLE_NAME as table_name')
                    .where('table_schema', knex.client.database());
                break;
            case KnexDriver_1.KnexDriver.ORACLE:
            case KnexDriver_1.KnexDriver.ORACLEDB:
                knexQuery = knex('user_tables')
                    .select('table_name');
                break;
            case KnexDriver_1.KnexDriver.PG:
                knexQuery = knex('information_schema.tables')
                    .select('table_name')
                    .where('table_schema', knex.raw('current_schema()'))
                    .andWhere('table_catalog', knex.client.database());
                break;
            case KnexDriver_1.KnexDriver.SQLITE3:
                knexQuery = knex('sqlite_master')
                    .select('name as table_name')
                    .where('type', 'table')
                    .andWhereNot('table_name', 'like', 'sqlite_%');
                break;
            default:
                throw new Error(`Driver "${knex.client.driverName}" not supported`);
        }
        return knexQuery.then((results) => {
            return results.map((row) => row.table_name);
        });
    }
    ping(knex) {
        let knexQuery;
        const limitResultsSize = 1;
        switch (knex.client.driverName) {
            case KnexDriver_1.KnexDriver.MSSQL:
                knexQuery = knex('information_schema.tables')
                    .select('table_name')
                    .where('table_schema', 'public')
                    .limit(limitResultsSize);
                break;
            case KnexDriver_1.KnexDriver.MYSQL:
            case KnexDriver_1.KnexDriver.MYSQL2:
                knexQuery = knex('information_schema.tables')
                    .select('TABLE_NAME as table_name')
                    .limit(limitResultsSize);
                break;
            case KnexDriver_1.KnexDriver.ORACLE:
            case KnexDriver_1.KnexDriver.ORACLEDB:
                knexQuery = knex('user_tables')
                    .select('table_name')
                    .limit(limitResultsSize);
                break;
            case KnexDriver_1.KnexDriver.PG:
                knexQuery = knex('information_schema.tables')
                    .select('table_name')
                    .limit(limitResultsSize);
                break;
            case KnexDriver_1.KnexDriver.SQLITE3:
                knexQuery = knex('sqlite_master')
                    .select('name as table_name')
                    .limit(limitResultsSize);
                break;
            default:
                throw new Error(`Driver "${knex.client.driverName}" not supported`);
        }
        return knexQuery;
    }
}
exports.DBTestManager = DBTestManager;
//# sourceMappingURL=DBTestManager.js.map